
PROGRAM toy_cnrmcm
  
  !! SS - oct 2017 - basé sur une contribution initiale d'A.Caubel

  !! Simule les aspects suivants de CNRM-CM
  !  - taille des grilles atmosphérique, riviere et océanique différenciées
  !  - traite un seul des contextes à la fois, en fonction du rang
  !  - grille atmosphérique en deux options : unstruct ou rectilineaire (et dans
  !      ce cas avec partie droite de la zone d'extension, si nexti>0)
  !  - grille riviere rectilineaire
  !  - taux de production (SYPD), par une boucle d'attente
  !  - champs 2D, avec une structure spatiale simple (codée par M-P.Moine), avec valeurs manquantes 
  !  - champs 3D atm de pression et height
  !
  !  - dans le context atmos, declare une grille rectilineaire nommé 'complete' et un axe
  !    de latitudes correspondant; la grille est distribuée de manière séparable. L'axe est distribué
  !    de manière redondante (chaque bande de latitudes est détenue par tous les procs qui ont une
  !    longitude à traiter dans cette bande)
  !
  !  - grilles oceaniques pour des moyennes zonales, avec et sans dimension 'basin' en sus
  !
  ! En sus : peut lancer n serveurs Xios (sur les taches surnuméraires) 
  !
  ! Mais : en MPMD, doit être lancé en premier


  

  ! exemple de namelist param.def :
  
  ! &param_toy

  ! duration=31d
  ! sypd=1000000
  ! startdate=1850-01-01

  ! atmid=surfex
  ! nb_proc_atm=160
  ! ni_atm=720
  ! nj_atm=360
  ! nlev_atm=91
  ! nlev_srf=14
  ! exti=0
  ! extj=0
  ! ntau=7
  ! nspectband=10
  ! ndbze=15
  ! nsza5=5
  ! nvegtype=6
  ! nlanduse=4
  ! nsoilpools=5 
  ! timestep_atm=900s
  
  ! rivid=trip
  ! nb_proc_riv=1
  ! ni_riv=360
  ! nj_riv=180
  ! timestep_riv=900s

  ! oceid=nemo
  ! nb_proc_oce=80
  ! ni_oce=1442
  ! nj_oce=1021
  ! nlev_oce=75
  ! nbasin_oce=4
  ! ntransects=4
  ! timestep_oce=900s
  ! !transect_names=
  ! /
  USE xios
  USE mod_wait
  IMPLICIT NONE
  INCLUDE "mpif.h"

  INTEGER,PARAMETER :: il_unit=10
  INTEGER :: comm, rank, size_loc, ierr
  INTEGER :: atm_rank, oce_rank
  INTEGER :: i,j,l,m,ts,n, nb_pt
  DOUBLE PRECISION :: sypd, timestep_in_seconds
  DOUBLE PRECISION :: simulated_seconds_per_seconds, elapsed_per_timestep
  DOUBLE PRECISION :: missing
  CHARACTER(len=*),PARAMETER :: id="client"
  CHARACTER(100)   :: atmid,oceid,rivid
  CHARACTER(10)    :: startdate
  CHARACTER(1000)  :: duration, timestep_atm, timestep_oce, timestep_riv
  TYPE(xios_date)  :: cdate, edate
  TYPE(xios_duration)  :: dtime
  TYPE(xios_context) :: ctx_hdl
  !
  REAL :: ilon,jlat,ztime, z0
  DOUBLE PRECISION,ALLOCATABLE :: lon_glo(:,:),lat_glo(:,:),lval(:), axis_bounds(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_A_glo (:,:,:), pressure_glo (:,:,:), height_glo (:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: bounds_lon_glo(:,:,:),bounds_lat_glo(:,:,:)
  !
  DOUBLE PRECISION,ALLOCATABLE :: field_riv_2D(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_2D(:),field_atm_3D(:,:),field_srf_2D(:),field_srf_3D(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_4D_tau(:,:,:), field_atm_4D_spectband(:,:,:), field_atm_4D_dbze(:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_4D_plev7_tau(:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_4D_cth16_tau(:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_4D_alt40_dbze(:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_4D_alt40_scatratio(:,:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_3D_sza5(:,:), field_atm_3D_vegtype(:,:), field_atm_3D_landuse(:,:), &
                                  field_atm_3D_soilpools(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_3D_temp(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_3D_alt40(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: other_field_atm_3D(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_atm_2D_miss(:)
  DOUBLE PRECISION,ALLOCATABLE :: pressure (:,:), height (:,:)
  DOUBLE PRECISION,ALLOCATABLE :: field_oce_2D(:,:),field_oce_3D(:,:,:), zonal_oce(:,:), zonal_basin_oce_2D(:,:,:), &
                                  zonal_basin_oce_1D(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: ap(:),b(:)
  !
  integer                      :: k,nk, kbegin,kend
  DOUBLE PRECISION,ALLOCATABLE :: lon(:,:),lat(:,:)
  DOUBLE PRECISION,ALLOCATABLE :: bounds_lon(:,:,:),bounds_lat(:,:,:) 
  DOUBLE PRECISION,ALLOCATABLE :: lon_1d(:),lat_1d(:)
  DOUBLE PRECISION,ALLOCATABLE :: bounds_lon_1d(:,:),bounds_lat_1d(:,:) 
  INTEGER, ALLOCATABLE         :: kindex(:),iindex(:),jindex(:), data_i_index(:)
  !
  LOGICAL :: ok
  !
  INTEGER,PARAMETER            :: nmax_transects=1000
  CHARACTER(100)               :: transect_names(nmax_transects)
  INTEGER                      :: ntransects, ibegin_transects, itransects_size
  DOUBLE PRECISION,ALLOCATABLE :: transects_field(:)
  !
  DOUBLE PRECISION, PARAMETER :: coef=2., dp_pi=3.14159265359
  DOUBLE PRECISION :: dp_length, dp_conv
  !
  INTEGER :: ni_atm, nj_atm,nlev_atm, nb_proc_atm, nlev_srf
  INTEGER :: ni_riv, nj_riv         , nb_proc_riv
  INTEGER :: ni_oce, nj_oce,nlev_oce, nb_proc_oce, nbasin_oce
  INTEGER :: nxi, nxj,nxlev ! 
  INTEGER :: nti, ntj
  LOGICAL :: distrib_lat
  !
  INTEGER :: exti, extj         ! Extension zone 'a la Aladin' (if exti >0)
  INTEGER :: nie_atm, nje_atm   ! sizes with extension
  INTEGER :: ntau               ! # of opticla thickness bands
  INTEGER :: nplev7             ! # ISCCP_PC (COSP)
  INTEGER :: nalt40             ! # of levels in statistical outputs (COSP)
  INTEGER :: ncth16             ! MISR_N_CTH (COSP)
  INTEGER :: nspectband         ! dim for spect band 
  INTEGER :: ndbze              ! # of bins for reflectivity
  INTEGER :: nsza5              ! # of particle size bins
  INTEGER :: ntemp              ! LIDAR_NTEMP (COSP)
  INTEGER :: nscatratio         ! # scatratio
  INTEGER :: nvegtype           ! dim 
  INTEGER :: nlanduse           ! dim 
  INTEGER :: nsoilpools         ! dim 
  INTEGER :: size_glo
  INTEGER :: offset, ibegin, ni, jbegin, nj,iend,jend
  !
  LOGICAL :: i_am_atm,i_am_oce, i_am_riv, i_am_server
  LOGICAL :: i_am_oce_leader, i_am_atm_leader, i_am_riv_leader, i_am_server_leader
  INTEGER :: oce_first_rank, atm_first_rank, riv_first_rank, server_first_rank
  INTEGER :: client_comm, color

  NAMELIST /param_toy/ &
       atmid,nb_proc_atm, ni_atm, nj_atm, nlev_atm, timestep_atm, exti,extj, nlev_srf, &
       ntau, nspectband, ndbze, nsza5, nvegtype, nlanduse, nsoilpools, distrib_lat, &
       nalt40, nplev7, ntemp, ncth16, nscatratio, &
       rivid,nb_proc_riv, ni_riv, nj_riv          , timestep_riv, &
       oceid,nb_proc_oce, ni_oce, nj_oce, nlev_oce, nbasin_oce, timestep_oce, &
       duration,sypd,startdate,transect_names

  !!!  Valeurs par défaut des paramètres du run

  timestep_atm="1d"
  timestep_riv="1d"
  timestep_oce="1d"
  nb_proc_atm=1 ; nlev_atm=10 ; nlev_srf=4 ;
  distrib_lat=.TRUE.

  ! These values are free
  nspectband=10 ;
  nvegtype=6 ;
  nsoilpools=5
  
  ! These values must be left unchanged if using the DR (tested with 01.00.21)
  ndbze=15 ;
  ntau=7 ;
  nsza5=5;
  nlanduse=3 ;
  nalt40 = 40
  nplev7 = 7
  ntemp = 40
  ncth16 = 16
  nscatratio = 15
  nbasin_oce=3 
  !
  !
  nb_proc_riv=0
  nb_proc_oce=0
  atmid="atmosphere"
  rivid="river"
  oceid="ocean"
  startdate="1950-01-01"
  exti=0 ! sera différent de 0 pour simuler Aladin
  extj=0
  transect_names=(/("", i=1,nmax_transects)/)
  transect_names(1:15)=(/("tr", i=1,15)/)

!!! Lecture des parametres du run

  OPEN(unit=il_unit, file='param.def',status='old',iostat=ierr)
  READ (il_unit, nml=param_toy)
  !PRINT *, ni_atm, nj_atm,nlev_atm,duration
  if (extj /= 0 ) then
     print *,'extj /= 0 is not handled'
     stop
  endif
  nie_atm=ni_atm+exti
  nje_atm=nj_atm+extj
  do i=0,nmax_transects
     if (transect_names(i) /= "" ) ntransects=i
  enddo

!!! MPI Initialization

  CALL MPI_INIT(ierr)
  CALL init_wait
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierr)
  CALL MPI_COMM_SIZE(MPI_COMM_WORLD,size_loc,ierr)
  if (size_loc < nb_proc_atm + nb_proc_oce + nb_proc_riv) then
     print *,"inconsistency : size=",size_loc," nb_proc_atm=",nb_proc_atm,&
          " nb_proc_oce=",nb_proc_oce," nb_proc_riv=",nb_proc_riv
     CALL MPI_ABORT()
  endif

  ! First ranks are dedicated to Ocean, next to Atm, next to River, last to Server
  oce_first_rank=0
  atm_first_rank=oce_first_rank+nb_proc_oce
  riv_first_rank=atm_first_rank+nb_proc_atm
  server_first_rank=riv_first_rank+nb_proc_riv

  i_am_oce=(nb_proc_oce>0) .and. (rank < nb_proc_oce)
  i_am_oce_leader=i_am_oce .and. (rank==oce_first_rank)
  if (i_am_oce) print*,"I am "//oceid

  i_am_atm=(nb_proc_atm>0) .and. (rank>=atm_first_rank) .and. (rank<riv_first_rank)
  i_am_atm_leader=i_am_atm .and. (rank == atm_first_rank)
  if (i_am_atm) print*,"I am "//atmid

  i_am_riv=(nb_proc_riv>0) .and. (rank>=riv_first_rank) .and. (rank<server_first_rank)
  i_am_riv_leader=i_am_riv .and. (rank == riv_first_rank)
  if (i_am_riv) print*,"I am "//rivid

  i_am_server=(rank >= server_first_rank)
  i_am_server_leader=i_am_server .and. (rank==server_first_rank)
  if (i_am_server) print*,"I am a server"

  call cpu_time(z0)
  if (i_am_server) then
     call xios_init_server()
  endif
  
!!! Initialisation et allocation des coordonnées et champs globaux
  ! Champs identiques pour atm et oce

  if (i_am_oce ) then
     nxi=ni_oce ; nxj=nj_oce ; nxlev=nlev_oce
     nti=ni_oce ; ntj=nj_oce 
  else if (i_am_atm) then 
     nxi=nie_atm  ; nxj=nje_atm  ; nxlev=nlev_atm
     nti=ni_atm ; ntj=nj_atm 
  else if (i_am_riv) then
     nxi=ni_riv ; nxj=nj_riv  ; nxlev=1
     nti=ni_riv ; ntj=nj_riv 
  endif

  if (.not. i_am_server) then 
     missing=1.e+20_8

     ALLOCATE (lon_glo(nxi,nxj),lat_glo(nxi,nxj))
     ALLOCATE (bounds_lon_glo(4,nxi,nxj))
     ALLOCATE (bounds_lat_glo(4,nxi,nxj))
     ALLOCATE (field_A_glo(nxi,nxj,nxlev))
     ALLOCATE (pressure_glo(nxi,nxj,nxlev))
     ALLOCATE (height_glo(nxi,nxj,nxlev))

     ! Parameters for analytical function
     dp_length= 1.2*dp_pi
     dp_conv=dp_pi/180.

     DO j=1,ntj
        DO i=1,nti

           ilon=i-0.5
           jlat=j-0.5

           lat_glo(i,j)= -90+(jlat*180./ntj)
           lon_glo(i,j)= (ilon*360./nti)

           bounds_lat_glo(1,i,j)= -90+((jlat-0.5)*180./ntj)
           bounds_lon_glo(1,i,j)=((ilon-0.5)*360./nti)

           bounds_lat_glo(2,i,j)= -90+((jlat-0.5)*180./ntj)
           bounds_lon_glo(2,i,j)=((ilon+0.5)*360./nti)

           bounds_lat_glo(3,i,j)= -90+((jlat+0.5)*180./ntj)
           bounds_lon_glo(3,i,j)=((ilon+0.5)*360./nti)

           bounds_lat_glo(4,i,j)= -90+((jlat+0.5)*180./ntj)
           bounds_lon_glo(4,i,j)=((ilon-0.5)*360./nti)

           DO l=1,nxlev
              field_A_glo(i,j,l)=(coef-SIN(dp_pi*(ACOS(COS(lat_glo(i,j)*dp_conv)*&
                   COS(lon_glo(i,j)*dp_conv))/dp_length)))+l
              pressure_glo(i,j,l)=((l-1.)/(nxlev-1.))*100000 + (jlat -ntj/2.)/ntj * 10000
              height_glo(i,j,l)=(nxlev-l+0.5)/nxlev * 15000 + jlat * 100
           ENDDO
        ENDDO
     ENDDO

     ! Use missing value for intializing Aladin E-zone

     DO j=1,nxj
        DO i=1,nxi
           DO l=1,nxlev
              if ((i>nti) .or. (j>ntj)) then
                 field_A_glo(i,j,l)=missing
                 pressure_glo(i,j,l)=missing
                 height_glo(i,j,l)=missing
                 !
                 lat_glo(i,j)= missing
                 lon_glo(i,j)= missing
                 !
                 bounds_lat_glo(1,i,j)=missing
                 bounds_lon_glo(1,i,j)=missing
                 bounds_lat_glo(2,i,j)=missing
                 bounds_lon_glo(2,i,j)=missing
                 bounds_lat_glo(3,i,j)=missing
                 bounds_lon_glo(3,i,j)=missing
                 bounds_lat_glo(4,i,j)=missing
                 bounds_lon_glo(4,i,j)=missing
              end if
           ENDDO
        ENDDO
     ENDDO
  endif

  !###########################################################################
  ! Contexte RIV
  !###########################################################################
  if (i_am_riv ) then 

!!! XIOS Initialization (get the local communicator)
     CALL xios_initialize("clients",return_comm=client_comm)
     color=0
     CALL MPI_COMM_RANK(client_comm,rank,ierr)
     CALL MPI_COMM_SPLIT(client_comm,color,rank,comm,ierr)

       
     CALL MPI_COMM_RANK(comm,rank,ierr)
     CALL MPI_COMM_SIZE(comm,size_loc,ierr)
     CALL xios_context_initialize(trim(rivid),comm)
     CALL xios_set_current_context(trim(rivid))

     ni=ni_riv ; ibegin=0 ; jbegin=0
     DO n=0,size_loc-1
        nj=nj_riv/size_loc
        IF (n<MOD(nj_riv,size_loc)) nj=nj+1
        IF (n==rank) exit
        jbegin=jbegin+nj
     ENDDO
     iend=ibegin+ni-1 ; jend=jbegin+nj-1

     ALLOCATE(lon(ni,nj),lat(ni,nj))
     lon(:,:)=lon_glo(ibegin+1:iend,jbegin+1:jend)
     lat(:,:)=lat_glo(ibegin+1:iend,jbegin+1:jend)

     ! Init fields
     ALLOCATE(field_riv_2D(ni,nj))
     field_riv_2D(1:ni,1:nj)=field_A_glo(ibegin+1:iend,jbegin+1:jend,1)

     CALL xios_set_domain_attr("domain_riv",ni_glo=ni_riv, nj_glo=nj_riv, ibegin=ibegin, &
          ni=ni,jbegin=jbegin,nj=nj, type='rectilinear')
     CALL xios_set_domain_attr("domain_riv",data_dim=2)
     CALL xios_set_domain_attr("domain_riv",lonvalue_1D=lon(:,1),latvalue_1D=lat(1,:))

     !!! Definition de la start date et du timestep
     call xios_set_start_date(xios_date_convert_from_string(startdate//" 00:00:00"))
     dtime=xios_duration_convert_from_string(timestep_riv)
     CALL xios_set_timestep(timestep=dtime)

     !!! Calcul de temps elaps par seconde pour respecter le SYPD (hyp : pas de délai d'I/O)
     CALL xios_get_start_date(cdate)
     edate=cdate+xios_duration_convert_from_string(duration)
     timestep_in_seconds=xios_date_convert_to_seconds(cdate+dtime) - xios_date_convert_to_seconds(cdate)
     simulated_seconds_per_seconds=sypd * 365 
     elapsed_per_timestep=timestep_in_seconds/simulated_seconds_per_seconds ! in seconds

     !!! Fin de la definition du contexte
     CALL xios_close_context_definition()
     CALL xios_set_current_context(trim(rivid))

     !####################################################################################
     !!! Boucle temporelle river
     !####################################################################################
     ts=1
     cdate=cdate+dtime
     DO while ( cdate <= edate )
        if (i_am_riv_leader ) print*,trim(rivid),' : ts=',ts
        !!! Mise a jour du pas de temps
        CALL xios_update_calendar(ts)
        !!! On donne la valeur du champ riv
        CALL xios_send_field("field_riv_2D",field_riv_2D+ts)
        !! Boucle d'attente
        CALL wait_us(int(elapsed_per_timestep*1.e6))   ! micro-secondes
        cdate=cdate+dtime
        ts=ts+1
     ENDDO

  endif


  !###########################################################################
  ! Contexte ATM
  !###########################################################################
  if (i_am_atm ) then

!!! XIOS Initialization (get the local communicator)
     CALL xios_initialize("clients",return_comm=client_comm)
     color=1
     CALL MPI_COMM_RANK(client_comm,rank,ierr)
     CALL MPI_COMM_SPLIT(client_comm,color,rank,comm,ierr)

     CALL MPI_COMM_RANK(comm,rank,ierr)
     CALL MPI_COMM_SIZE(comm,size_loc,ierr)
     CALL xios_context_initialize(trim(atmid),comm)
     CALL xios_get_handle(trim(atmid),ctx_hdl)
     CALL xios_set_current_context(ctx_hdl)
     !CALL xios_set_current_context(trim(atmid))
     atm_rank=rank ! rang DANS LE CONTEXTE ATM
     !print *,'I am atm, rank=',rank, ' atm_rank=',atm_rank

     ! Compute task 1D index range

     ! Same as Aladin : send only right part of extension zone (not upper one)
     size_glo=nie_atm*nj_atm
     nk=size_glo/nb_proc_atm;
     if (mod(size_glo,nb_proc_atm)>0) nk=nk+1
     kbegin=atm_rank*nk ; !commence à 0 - convention C
     if (kbegin >= size_glo) kbegin=size_glo-1
     kend=kbegin+nk-1 ;
     if (kend >= size_glo) then
        kend=size_glo-1
        nk=kend-kbegin+1
     endif

     ! Init local lat and lon arrays

     allocate(kindex(nk), iindex(nk), jindex(nk)) 
     ALLOCATE(lon_1d(nk),lat_1d(nk),bounds_lon_1d(4,nk),bounds_lat_1d(4,nk), data_i_index(nk))
     do k=0,nk-1
        i= mod((kbegin+k),nie_atm) +1
        j= (kbegin+k)/nie_atm + 1
        if ((i<=nie_atm) .and. (j<=nj_atm)) then
           iindex(k+1)=i-1 ; jindex(k+1)=j-1 ! for rectilinear/curvilinear domain
           kindex(k+1)=kbegin+k ! for unstruct domain
           lon_1d(k+1)=lon_glo(i,j)
           lat_1d(k+1)=lat_glo(i,j)
           bounds_lon_1d(:,k+1)=bounds_lon_glo(:,i,j)
           bounds_lat_1d(:,k+1)=bounds_lat_glo(:,i,j)
        else
           print *, 'logic error' ; stop
        end if
     end do

     ALLOCATE (lval(nlev_atm),axis_bounds(2,nlev_atm))
     DO i=1,nlev_atm
        lval(i)=(i+0.)/nlev_atm
        axis_bounds(1,i)=(i-0.5)/nlev_atm
        axis_bounds(2,i)=(i+0.5)/nlev_atm
     ENDDO
     if (distrib_lat) then 
        CALL SFX_XIOS_GRID(nj_atm,ni_atm,rank-atm_first_rank,nb_proc_atm)
     else
        CALL xios_set_domain_attr("complete",type="rectilinear",ni_glo=ni_atm , nj_glo=nj_atm )
        CALL xios_set_axis_attr("glat",n_glo=nj_atm, n=nj_atm, value=lat_glo(1,1:nj_atm))
     endif
     ! print *,'calling xios_grid for relative atm rank=',rank-atm_first_rank ; call flush(6)
     CALL xios_set_axis_attr("axis_atm",n_glo=nlev_atm ,value=lval, bounds=axis_bounds) ;
     DEALLOCATE (lval)
     ALLOCATE (lval(nlev_srf))
     DO i=1,nlev_srf
        lval(i)=i
     ENDDO
     CALL xios_set_axis_attr("axis_srf",n_glo=nlev_srf ,value=lval) ;
     !
     CALL xios_set_axis_attr("axis_tau",n_glo=ntau ) ;
     CALL xios_set_axis_attr("axis_spectband",n_glo=nspectband ) ;
     CALL xios_set_axis_attr("axis_dbze",n_glo=ndbze ) ;
     CALL xios_set_axis_attr("axis_sza5",n_glo=nsza5 ) ;
     CALL xios_set_axis_attr("axis_temp",n_glo=ntemp ) ;
     CALL xios_set_axis_attr("axis_cth16",n_glo=ncth16 ) ;
     CALL xios_set_axis_attr("axis_scatratio",n_glo=nscatratio ) ;
     CALL xios_set_axis_attr("axis_alt40",n_glo=nalt40 ) ;
     CALL xios_set_axis_attr("axis_plev7",n_glo=nplev7 ) ;
     CALL xios_set_axis_attr("axis_vegtype",n_glo=nvegtype ) ;
     CALL xios_set_axis_attr("axis_landuse",n_glo=nlanduse ) ;
     CALL xios_set_axis_attr("axis_soilpools",n_glo=nsoilpools ) ;

     ! Init fields

     ALLOCATE(field_atm_2D(nk),field_atm_3D(nk,nlev_atm))
     ALLOCATE(field_atm_4D_tau(nk,nlev_atm,ntau), field_atm_4D_spectband(nk,nlev_atm,nspectband), &
              field_atm_4D_dbze(nk,nlev_atm,ndbze))
     ALLOCATE(field_atm_4D_plev7_tau(nk, nplev7, ntau))
     ALLOCATE(field_atm_4D_cth16_tau(nk, ncth16, ntau))
     ALLOCATE(field_atm_4D_alt40_dbze(nk, nalt40, ndbze))
     ALLOCATE(field_atm_4D_alt40_scatratio(nk, nalt40, nscatratio))
     ALLOCATE(field_atm_3D_sza5(nk,nsza5 ),field_atm_3D_vegtype(nk,nvegtype ))
     ALLOCATE(field_atm_3D_temp(nk, ntemp))
     ALLOCATE(field_atm_3D_alt40(nk, nalt40))
     ALLOCATE(field_atm_3D_landuse(nk,nlanduse),field_atm_3D_soilpools(nk,nsoilpools))
     ALLOCATE(field_srf_2D(nk),field_srf_3D(nk,nlev_srf))
     ALLOCATE(other_field_atm_3D(nk,nlev_atm))
     ALLOCATE(field_atm_2D_miss(nk))
     ALLOCATE(pressure(nk,nlev_atm))
     ALLOCATE(height(nk,nlev_atm))
     !
     do k=0,nk-1
        i= mod(kbegin+k,nie_atm) +1
        j= (kbegin+k)/nie_atm + 1 ;
        field_atm_2D(k+1)     =field_A_glo(i,j,1)
        field_srf_2D(k+1)     =field_A_glo(i,j,1)
        field_atm_2D_miss(k+1)=field_A_glo(i,j,1)
        field_atm_3D(k+1,:)   =field_A_glo(i,j,:)
        do l=1,ntau
           field_atm_4D_tau(k+1,:,l) =field_A_glo(i,j,:)+(l-1)*100
        enddo
        do l=1,ntau
           do m=1,nplev7
             field_atm_4D_plev7_tau(k+1,m,l) =field_A_glo(i,j,1)+(l-1)*100
           enddo
        enddo
        do l=1,ntau
           do m=1,ncth16
             field_atm_4D_cth16_tau(k+1,m,l) =field_A_glo(i,j,1)+(l-1)*100
           enddo
        enddo
        do l=1,ndbze
           do m=1,nalt40
             field_atm_4D_alt40_dbze(k+1,m,l) =field_A_glo(i,j,1)+(l-1)*100
           enddo
        enddo
        do l=1,nscatratio
           do m=1,nalt40
             field_atm_4D_alt40_scatratio(k+1,m,l) =field_A_glo(i,j,1)+(l-1)*100
           enddo
        enddo
        do l=1,nspectband
           field_atm_4D_spectband(k+1,:,l) =field_A_glo(i,j,:)+(l-1)*100
        enddo
        do l=1,ndbze
           field_atm_4D_dbze(k+1,:,l) =field_A_glo(i,j,:)+(l-1)*100
        enddo
        do l=1,nsza5
           field_atm_3D_sza5(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        do l=1,ntemp
           field_atm_3D_temp(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        do l=1, nalt40
           field_atm_3D_alt40(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        do l=1,nvegtype
           field_atm_3D_vegtype(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        do l=1,nlanduse
           field_atm_3D_landuse(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        do l=1,nsoilpools
           field_atm_3D_soilpools(k+1,l) =field_A_glo(i,j,1)+(l-1)*100
        enddo
        field_srf_3D(k+1,1:nlev_srf)=field_A_glo(i,j,1:nlev_srf)
        pressure(k+1,:)       =pressure_glo(i,j,:)
        height(k+1,:)         =height_glo(i,j,:)
     end do

     ALLOCATE (ap(nlev_atm),b(nlev_atm))
     DO i=1,nlev_atm
        ap(i)=101325.*(i-1.)/(nlev_atm-1.)
        b(i)=(i-1.)/(nlev_atm-1.)
     ENDDO


!!! declare atmospheric domain

     if (exti==0) then
        ! We are in case of global grid, declared as unstruct

        CALL xios_set_domain_attr("domain_atm", data_dim=1)
        CALL xios_set_domain_attr("domain_atm", type='unstructured',ni_glo=size_glo)
        !
        CALL xios_set_domain_attr("domain_atm", ibegin=0, ni=nk, i_index=kindex )
        CALL XIOS_SET_DOMAIN_ATTR("domain_atm", lonvalue_1d=lon_1d,latvalue_1d=lat_1d)
        CALL XIOS_SET_DOMAIN_ATTR("domain_atm", nvertex=4, bounds_lat_1d=bounds_lat_1d, &
                                  bounds_lon_1d=bounds_lon_1d)

     else
        ! LAM Aladin case, with an extension zone to the right of the physical zone
        offset=mod((kbegin),nie_atm)
        DO k=0,nk-1
           i= mod((kbegin+k),nie_atm)
           IF (i<nxi) THEN
              data_i_index(k+1)=offset
              offset=offset+1
           ELSE
              data_i_index(k+1)=-1
           ENDIF
        ENDDO

        ibegin=0 ; ni=nxi
        jbegin=jindex(1) ; nj=jindex(nk)-jindex(1)+1

        CALL xios_set_domain_attr("domain_atm", data_dim=1)
        CALL xios_set_domain_attr("domain_atm", type='rectilinear',ni_glo=nxi)
        CALL xios_set_domain_attr("domain_atm", nj_glo=nj_atm)
        !
        CALL xios_set_domain_attr("domain_atm", ibegin=ibegin, ni=ni)
        CALL XIOS_SET_DOMAIN_ATTR("domain_atm", jbegin=jbegin,nj=nj)
        CALL XIOS_SET_DOMAIN_ATTR("domain_atm", data_ni=nk, data_i_index=data_i_index)
        CALL XIOS_SET_DOMAIN_ATTR("domain_atm", lonvalue_1d=lon_glo(1:nxi,1),&
                                  latvalue_1d=lat_glo(1,jbegin+1:jbegin+nj))
     endif

     !!! Definition de la start date et du timestep
     call xios_set_start_date(xios_date_convert_from_string(startdate//" 00:00:00"))
     dtime=xios_duration_convert_from_string(timestep_atm)
     CALL xios_set_timestep(timestep=dtime)

     !!! Calcul de temps elaps par seconde pour respecter le SYPD (hyp : pas de délai d'I/O)
     CALL xios_get_start_date(cdate)
     edate=cdate+xios_duration_convert_from_string(duration)
     timestep_in_seconds=xios_date_convert_to_seconds(cdate+dtime) - xios_date_convert_to_seconds(cdate)
     simulated_seconds_per_seconds=sypd * 365 
     elapsed_per_timestep=timestep_in_seconds/simulated_seconds_per_seconds ! in seconds

     !!! Fin de la definition du contexte
     CALL xios_close_context_definition()
     CALL xios_set_current_context(trim(atmid))
     if (i_am_atm_leader ) then
        call cpu_time(ztime)
        print*,trim(atmid),' xios init duration=',ztime-z0
        z0=ztime
     endif

     !####################################################################################
     !!! Boucle temporelle atmosphere
     !####################################################################################
     ts=1
     cdate=cdate+dtime
     DO while ( cdate <= edate )
        !!! Mise a jour du pas de temps
        CALL xios_update_calendar(ts)

        !!! On donne la valeur des champ atm
        CALL xios_send_field("field_atm_scalar",field_atm_2D(1)+ts)
        CALL xios_send_field("field_atm_1D",field_atm_3D(1,:)+ts)
        CALL xios_send_field("field_atm_2D",field_atm_2D+ts)
        other_field_atm_3D(:,:)=field_atm_3D(:,:)+ts
        CALL xios_send_field("other_field_atm_3D",field_atm_3D)
        CALL xios_send_field("field_atm_3D",field_atm_3D)
        CALL xios_send_field("field_atm_4D_tau",field_atm_4D_tau)
        CALL xios_send_field("field_atm_4D_spectband",field_atm_4D_spectband)
        CALL xios_send_field("field_atm_4D_plev7_tau",field_atm_4D_plev7_tau)
        CALL xios_send_field("field_atm_4D_cth16_tau",field_atm_4D_cth16_tau)
        CALL xios_send_field("field_atm_4D_alt40_dbze",field_atm_4D_alt40_dbze)
        CALL xios_send_field("field_atm_4D_alt40_scatratio",field_atm_4D_alt40_scatratio)
        CALL xios_send_field("field_atm_4D_dbze",field_atm_4D_dbze)
        CALL xios_send_field("field_atm_3D_vegtype",field_atm_3D_vegtype)
        CALL xios_send_field("field_atm_3D_landuse",field_atm_3D_landuse)
        CALL xios_send_field("field_atm_3D_soilpools",field_atm_3D_soilpools)
        CALL xios_send_field("field_atm_3D_sza5",field_atm_3D_sza5)
        CALL xios_send_field("field_atm_3D_temp",field_atm_3D_temp)
        CALL xios_send_field("field_atm_3D_alt40",field_atm_3D_alt40)
        CALL xios_send_field("pressure" ,pressure)
        CALL xios_send_field("height"   ,height)
        if (mod(ts,2)==0) then
           CALL xios_send_field("field_sub",field_atm_2D+ts)
        endif
        !! On crée un champ avec des missings qui bougent
        !! dans le temps : un bande verticale de 1 à ni-3
        field_atm_2D_miss(:)= field_atm_2D(:)+ts
        field_atm_2D_miss(mod(ts,nk-3)+1)=missing
        CALL xios_send_field("field_miss",field_atm_2D_miss)

        CALL xios_send_field("ap",ap)
        CALL xios_send_field("b",b)

!!! On donne la valeur du champ srf
        CALL xios_send_field("field_srf_2D",field_srf_2D)
        CALL xios_send_field("field_srf_3D",field_srf_3D)

        !! Boucle d'attente
        CALL wait_us(int(elapsed_per_timestep*1.e6))   ! micro-secondes
        !
        if (i_am_atm_leader ) then
           call cpu_time(ztime)
           print*,trim(atmid),' : ts=',ts,' delta=',ztime-z0
           z0=ztime
        endif
        !
        cdate=cdate+dtime
        ts=ts+1
     ENDDO

  endif


  if (i_am_oce) then 

     !###########################################################################
     ! Contexte OCE
     !###########################################################################
     CALL xios_initialize("clients",return_comm=client_comm)
     color=2
     CALL MPI_COMM_RANK(client_comm,rank,ierr)
     CALL MPI_COMM_SPLIT(client_comm,color,rank,comm,ierr)
     
     CALL MPI_COMM_RANK(comm,rank,ierr)
     CALL MPI_COMM_SIZE(comm,size_loc,ierr)
     CALL xios_context_initialize(trim(oceid),comm)
     CALL xios_set_current_context(trim(oceid))

     ni=ni_oce ; ibegin=0

     jbegin=0
     DO n=0,nb_proc_oce-1
        nj=nj_oce/nb_proc_oce
        IF (n<MOD(nj_oce,nb_proc_oce)) nj=nj+1
        IF (n==rank) exit
        jbegin=jbegin+nj
     ENDDO
     iend=ibegin+ni-1 ; jend=jbegin+nj-1

     ALLOCATE(lon(ni,nj),lat(ni,nj))
     ALLOCATE(bounds_lon(4,ni,nj))
     ALLOCATE(bounds_lat(4,ni,nj))
     lon(:,:)=lon_glo(ibegin+1:iend+1,jbegin+1:jend+1)
     lat(:,:)=lat_glo(ibegin+1:iend+1,jbegin+1:jend+1)
     bounds_lon(:,:,:)=bounds_lon_glo(:,ibegin+1:iend+1,jbegin+1:jend+1)
     bounds_lat(:,:,:)=bounds_lat_glo(:,ibegin+1:iend+1,jbegin+1:jend+1)

     ALLOCATE(field_oce_2D(ni,nj),field_oce_3D(ni,nj,nlev_oce))
     field_oce_2D(1:ni,1:nj)=field_A_glo(ibegin+1:iend+1,jbegin+1:jend+1,1)
     field_oce_3D(1:ni,1:nj,:)=field_A_glo(ibegin+1:iend+1,jbegin+1:jend+1,:)

     ALLOCATE (lval(nlev_oce))
     DO i=1,nlev_oce
        lval(i)=i
     ENDDO

     ALLOCATE(zonal_oce(nj,nlev_oce),zonal_basin_oce_2D(nj,nlev_oce,nbasin_oce), &
              zonal_basin_oce_1D(nj,nbasin_oce))
     ALLOCATE(transects_field(ntransects))
     zonal_oce(:,:)=field_oce_3D(1,:,:)
     do i=1,nbasin_oce
        zonal_basin_oce_2D(:,:,i)=field_oce_3D(1,:,:)+i
        zonal_basin_oce_1D(:,i)=field_oce_2D(1,:)+i
     enddo
     do i=1,ntransects
        transects_field(i) = i
     enddo
        
     
     CALL xios_set_axis_attr("axis_oce",n_glo=nlev_oce ,value=lval) ;
     if (xios_is_valid_axis("lat_oce")) then
        print *,'declaring axis lat_oce, n_glo=',nj_oce,'n=',nj,' size(value)=',size(lat,2)
        CALL xios_set_axis_attr("lat_oce" ,n_glo=nj_oce ,n=nj, value=lat(1,:))
     endif
     if (xios_is_valid_axis("basin_oce")) then
        CALL xios_set_axis_attr("basin_oce" ,n_glo=nbasin_oce)
     endif

     CALL xios_set_domain_attr("domain_oce",ni_glo=ni_oce, nj_glo=nj_oce, ibegin=ibegin, ni=ni,&
                               jbegin=jbegin,nj=nj, type='curvilinear')
     CALL xios_set_domain_attr("domain_oce",data_dim=2)!, data_ibegin=-1, data_ni=ni+2, data_jbegin=-2,&
                                                       ! data_nj=nj+4)
     CALL xios_set_domain_attr("domain_oce",lonvalue_2D=lon,latvalue_2D=lat)
     CALL xios_set_domain_attr("domain_oce",nvertex=4, bounds_lon_2d=bounds_lon, bounds_lat_2d=bounds_lat)

     !!! compute transects local range (in case we actually distribute this axis)
     ibegin_transects=0
     DO n=0,nb_proc_oce-1
        itransects_size=ntransects/nb_proc_oce
        IF (n<MOD(ntransects,nb_proc_oce)) itransects_size=itransects_size+1
        IF (n==rank) exit
        ibegin_transects=ibegin_transects+itransects_size
     ENDDO

     if (xios_is_valid_axis("transect_axis") .and. (ntransects > 0)) then
        CALL xios_set_axis_attr("transect_axis",n_glo=ntransects)

        ! Version with a distributed transect axis
        ! CALL xios_set_axis_attr("transect_axis",begin=ibegin_transects,n=itransects_size)
        ! CALL xios_set_axis_attr("transect_axis",label=transect_names(&
        !                ibegin_transects:ibegin_transects+itransects_size))

        ! Version with a non-distributed transect axis
        !print *,'initalizing labels for transect_axis to:',transect_names
        CALL xios_set_axis_attr("transect_axis",value=(/(1.0D0*I, i=1,ntransects)/))
        CALL xios_set_axis_attr("transect_axis",label=transect_names(1:ntransects))
        CALL xios_set_axis_attr("transect_axis",name="oline")
        CALL xios_set_axis_attr("transect_axis",standard_name="Region")
        CALL xios_set_axis_attr("transect_axis",long_name="ocean passage")
     endif

     !!! Definition de la start date et du timestep
     call xios_set_start_date(xios_date_convert_from_string(startdate//" 00:00:00"))
     dtime=xios_duration_convert_from_string(timestep_oce)
     CALL xios_set_timestep(timestep=dtime)

     !!! Calcul de temps elaps par seconde pour respecter le SYPD (hyp : pas de délai d'I/O)
     CALL xios_get_start_date(cdate)
     edate=cdate+xios_duration_convert_from_string(duration)
     timestep_in_seconds=xios_date_convert_to_seconds(cdate+dtime) - xios_date_convert_to_seconds(cdate)
     simulated_seconds_per_seconds=sypd * 365 
     elapsed_per_timestep=timestep_in_seconds/simulated_seconds_per_seconds ! in seconds

     !!! Fin de la definition du contexte
     CALL xios_close_context_definition()
     CALL xios_set_current_context(trim(oceid))

     !####################################################################################
     !!! Boucle temporelle ocean
     !####################################################################################
     ts=1
     cdate=cdate+dtime
     DO while ( cdate <= edate )

        if (i_am_oce_leader ) print*,trim(oceid),' : ts=',ts
        !!! Mise a jour du pas de temps
        CALL xios_update_calendar(ts)

        !!! On donne la valeur des champs oce
        CALL xios_send_field("field_oce_scalar",field_oce_2D(1,1)+ts)
        CALL xios_send_field("field_oce_2D",field_oce_2D)
        CALL xios_send_field("field_oce_3D",field_oce_3D)
        if (xios_is_valid_field("transect_field") .and. (ntransects > 0)) then
           ! Version with a distributed transect axis
           ! call xios_send_field("transect_field",transects_field(ibegin_transects:ibegin_transects+&
           !                                                       itransects_size))
           ! Version with a non-distributed transect axis
           call xios_send_field("transect_field",transects_field)
        endif
        if (xios_is_valid_field("zonal_basin_oce_2D")) then
           CALL xios_send_field("zonal_basin_oce_2D",zonal_basin_oce_2D)
        endif
        if (xios_is_valid_field("zonal_basin_oce_1D")) then
           CALL xios_send_field("zonal_basin_oce_1D",zonal_basin_oce_1D)
        endif
        if (xios_is_valid_field("zonal_oce")) then
           CALL xios_send_field("zonal_oce",zonal_oce)
        endif
        !
        CALL wait_us(int(elapsed_per_timestep*1.e6))   ! micro-secondes
        cdate=cdate+dtime
        ts=ts+1

     ENDDO

  endif

  !!! Fin de XIOS
  if ( .not. i_am_server) then
     CALL MPI_BARRIER(client_comm,ierr)
     CALL xios_context_finalize()
     CALL MPI_COMM_FREE(comm, ierr)
     CALL xios_finalize()
  endif

  CALL MPI_FINALIZE(ierr)




END PROGRAM toy_cnrmcm





SUBROUTINE SFX_XIOS_GRID(KDGLG,KDLON,KRANK,KPROC)
!!
!!
!!     PURPOSE : declare to XIOS a distribution for rectilinear grid 'complete'
!!     --------  and for a corresponding latitudes axis
!!
!!
!!     METHOD
!!     ------
!!
!!     Based on number of latitudes and longitudes, and proc number,
!!     compute a rectilinear distribution for a rectilinear grid
!!
!!     Some MPI tasks may be at rest, for the sake of the rectilinear 
!!     distribution
!!
!!     AUTHOR     S.Sénési, CNRM
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
!
USE XIOS     , ONLY : XIOS_DOMAIN, XIOS_DOMAINGROUP,XIOS_SET_DOMAIN_ATTR, &
                      XIOS_AXIS, XIOS_AXISGROUP,XIOS_SET_AXIS_ATTR,&
                      XIOS_ADD_CHILD, XIOS_GET_HANDLE
!
IMPLICIT NONE
!
INTEGER, INTENT(IN) :: KDGLG         ! number of rows of latitudes
INTEGER, INTENT(IN) :: KDLON         ! length of a row of latitude
INTEGER, INTENT(IN) :: KRANK         ! rank of mpi task (from 0)
INTEGER, INTENT(IN) :: KPROC         ! number of procs
!
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
! CHARACTER(LEN=8), PARAMETER       :: YNAMGRID="complete"
! CHARACTER(LEN=4), PARAMETER       :: YNAMAXIS="glat"
INTEGER                           :: NI,NJ,I,J,IOFF,JOFF,NEXT
INTEGER                           :: NREG_NS,NREG_EW 
INTEGER                           :: NREGS_NS,NREGS_EW 
INTEGER                           :: IRESIDUAL, NX_NS, NX_EW

REAL                   :: ZINCR
REAL                   :: FACT        
DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:)   :: ZLATI,ZLONG
DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: ZLATIC,ZLONGC
!
TYPE(xios_domaingroup)            :: domaingroup_hdl
TYPE(xios_domain)                 :: domain_hdl
TYPE(xios_axisgroup)              :: axisgroup_hdl
TYPE(xios_axis)                   :: axis_hdl
!
! Basic XIOS declarations
 CALL XIOS_GET_HANDLE("domain_definition",domaingroup_hdl)
 CALL XIOS_ADD_CHILD(domaingroup_hdl,domain_hdl,"complete")
 CALL XIOS_SET_DOMAIN_ATTR("complete", type="rectilinear", data_dim=2)
 !
 ! Choose a separable NW/SE distribution for  grid,'complete' based
 ! on grid size and proc number. Method : explore all couples of
 ! NREGS_NS, NREGS_EW which match NREGS_NS * NREGS_EW <= KPROC and
 ! choose the one with minimal residual and minimal
 ! abs(NREGS_EW-2*NREGS_NS) in order to get somewhat 'square' regions
 !
 ! The # of unused procs is less than 1 for kproc < 20.000
IRESIDUAL=KPROC
NREGS_NS=1
NREGS_EW=KPROC
DO J=KPROC-1,2,-1
   DO I=KPROC-1,2,-1
      IF ((I*J <= KPROC) .AND. ((KPROC-I*J) <= IRESIDUAL  )) THEN 
         IF ((ABS(2*I-J) < ABS(2*NREGS_NS-NREGS_EW)) .OR. &
              (IRESIDUAL == KPROC))  THEN
            IRESIDUAL = KPROC -I*J
            NREGS_NS=I
            NREGS_EW=J
         ENDIF
      ENDIF
   ENDDO
ENDDO
!
! Compute region indices (from 1) for current MPI task
!
NREG_NS = 1+ KRANK/NREGS_EW 
NREG_EW = 1 + mod(KRANK,NREGS_EW)
!
! Compute domain size in longitude 
!
NEXT=0
IF (MOD(KDLON,NREGS_EW)==0) THEN
  NI=KDLON/NREGS_EW
  IOFF=(NREG_EW-1)*NI
ELSE
  NI=KDLON/NREGS_EW+1
  IF (NI*(NREGS_EW-1) < KDLON) THEN
    IOFF=(NREG_EW-1)*NI
    IF (NREG_EW==NREGS_EW) NI=MOD(KDLON,NI)
  ELSE
    NI=KDLON/NREGS_EW
    NEXT=MOD(KDLON,NREGS_EW)
    IF (NREG_EW <= NEXT) THEN
      NI=NI+1
      IOFF=(NREG_EW-1)*NI
    ELSE
      IOFF=(NREG_EW-1)*NI+NEXT
    ENDIF
  ENDIF
ENDIF
!
! Compute domain size in latitude 
!
NEXT=0
IF (MOD(KDGLG,NREGS_NS)==0) THEN
  NJ=KDGLG/NREGS_NS
  JOFF=(NREG_NS-1)*NJ
ELSE
  NJ=KDGLG/NREGS_NS+1
  IF (NJ*(NREGS_NS-1) < KDGLG) THEN
    JOFF=(NREG_NS-1)*NJ
    IF (NREG_NS==NREGS_NS) NJ=MOD(KDGLG,NJ)
  ELSE
    NJ=KDGLG/NREGS_NS
    NEXT=MOD(KDGLG,NREGS_NS)
    IF (NREG_NS <= NEXT) THEN
      NJ=NJ+1
      JOFF=(NREG_NS-1)*NJ
    ELSE
      JOFF=(NREG_NS-1)*NJ+NEXT
    ENDIF
  ENDIF
ENDIF
!
! Use dummy values for tasks which are unused (due to rectilinear
! distribution)
!
IF (KRANK >= NREGS_NS*NREGS_EW) THEN
   NI=0
   IOFF=1
   NJ=0
   JOFF=1
ENDIF
!
! Compute evenly spaced longitudes
!
ALLOCATE(ZLONG(NI))
ALLOCATE(ZLONGC(2,NI))
ZINCR=360./KDLON
DO I=1,NI
   ZLONG(I)=(IOFF+I-1)*ZINCR ! Choose to have longitude 0 at grid mesh center
   !ZLONG(I)=(IOFF+I-0.5)*ZINCR
   ZLONGC(1,I)=ZLONG(I) - ZINCR/2.
   ZLONGC(2,I)=ZLONG(I) + ZINCR/2.
ENDDO
!
 CALL XIOS_SET_DOMAIN_ATTR("complete", ni_glo=KDLON, ni=NI,ibegin=IOFF)
 CALL XIOS_SET_DOMAIN_ATTR("complete", lonvalue_1d=ZLONG(1:NI), bounds_lon_1D=ZLONGC(:,1:NI))
!
!
ALLOCATE(ZLATI(NJ))
ALLOCATE(ZLATIC(2,NJ))
ZINCR=180./KDGLG
!
DO J=1,NJ
   ZLATI(J)=-90. + (JOFF+J-0.5)*ZINCR
   ZLATIC(1,J)=ZLATI(J) - ZINCR/2.
   ZLATIC(2,J)=ZLATI(J) + ZINCR/2.
ENDDO
!
!
 CALL XIOS_SET_DOMAIN_ATTR("complete", nj_glo=KDGLG, nj=NJ, jbegin=JOFF)
 CALL XIOS_SET_DOMAIN_ATTR("complete",latvalue_1d=ZLATI(1:NJ),bounds_lat_1D=ZLATIC(:,1:NJ))
 !print "(I3,I3,'/',I3,' , ',I3,'/',I3,' ni=',I2,' nj=',I2)",krank, nreg_ns,nregs_ns,nreg_ew,&
 !       nregs_ew,ni,nj ; call flush(6)
!
!
! Define and distribute an axis for latitudes
!
!
 CALL XIOS_GET_HANDLE("axis_definition",axisgroup_hdl)
 CALL XIOS_ADD_CHILD(axisgroup_hdl,axis_hdl,"glat")
 CALL XIOS_SET_AXIS_ATTR("glat", standard_name="latitude", unit="degrees_north")
 CALL XIOS_SET_AXIS_ATTR("glat", n_glo=KDGLG, begin=JOFF, n=NJ)
 CALL XIOS_SET_AXIS_ATTR("glat", value=ZLATI(1:NJ))
 CALL XIOS_SET_AXIS_ATTR("glat", bounds=ZLATIC(:,:))
!
!
!
DEALLOCATE(ZLATI,ZLONG,ZLATIC,ZLONGC)
END SUBROUTINE SFX_XIOS_GRID

