!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     #########
      SUBROUTINE DIAG_MISC_ISBA_INIT_n (DU, DM, NDM, IO, NP, HSNOW_SCHEME, KSNOW_NLAYER, KLU, KSW)
!     #####################
!
!!****  *DIAG_MISC_ISBA_INIT_n* - routine to initialize ISBA misc diagnostic variables
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!      V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      B. Decharme    02/2022 : split misc diag from DIAG_ISBA_INIT_n
!!
!-------------------------------------------------------------------------------
!
!*       0.0    DECLARATIONS
!              ------------
!
USE MODD_DIAG_MISC_ISBA_n, ONLY : DIAG_MISC_ISBA_t, DIAG_MISC_ISBA_NP_t
USE MODD_ISBA_OPTIONS_n,   ONLY : ISBA_OPTIONS_t
USE MODD_ISBA_n,           ONLY : ISBA_NP_t
USE MODD_DIAG_UTCI_n,      ONLY : DIAG_UTCI_t
!
USE MODD_UTCI,             ONLY : NUTCI_STRESS
USE MODD_SURF_PAR,         ONLY : XUNDEF
USE MODD_PREP_SNOW,        ONLY : NIMPUR
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.01   Declarations of arguments
!              -------------------------
!
TYPE(DIAG_UTCI_t),         INTENT(INOUT) :: DU
TYPE(DIAG_MISC_ISBA_t),    INTENT(INOUT) :: DM
TYPE(DIAG_MISC_ISBA_NP_t), INTENT(INOUT) :: NDM
TYPE(ISBA_OPTIONS_t),      INTENT(INOUT) :: IO
TYPE(ISBA_NP_t),           INTENT(INOUT) :: NP
!
CHARACTER(LEN=*), INTENT(IN) :: HSNOW_SCHEME
INTEGER,          INTENT(IN) :: KSNOW_NLAYER
INTEGER,          INTENT(IN) :: KLU       ! size of arrays
INTEGER,          INTENT(IN) :: KSW       ! spectral bands
!
!*       0.02   Declarations of local variables
!              -------------------------------
!
INTEGER         :: JP, INIP
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N',0,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
!
!* miscellaneous surface fields
!
CALL ALLOC_MISC_BUD(DM, KLU ,0)
!
!* Crocus specific
!
IF (HSNOW_SCHEME=="CRO" ) THEN
   CALL ALLOC_MISC_CROCUS(DM, KLU ,0 ,DM%LPROSNOW)
ELSE
   CALL ALLOC_MISC_CROCUS(DM, 0 ,0 ,DM%LPROSNOW)
ENDIF
!
!* UTCI specific
!
IF (DU%LUTCI) THEN
   CALL ALLOC_MISC_UTCI(DU, KLU)
ELSE
   CALL ALLOC_MISC_UTCI(DU, 0)
ENDIF
!
!
!-------------------------------------------------------------------------------
!
!* miscellaneous surface fields per patch
!
DO JP=1,IO%NPATCH
   INIP = NP%AL(JP)%NSIZE_P   
   CALL ALLOC_MISC_BUD(NDM%AL(JP), INIP, INIP)
ENDDO
!
!* Crocus specific per patch
!
IF (HSNOW_SCHEME=="CRO" ) THEN
   DO JP=1,IO%NPATCH
      INIP = NP%AL(JP)%NSIZE_P   
      CALL ALLOC_MISC_CROCUS(NDM%AL(JP), INIP, INIP, DM%LPROSNOW)
   ENDDO
ELSE
   DO JP=1,IO%NPATCH
      CALL ALLOC_MISC_CROCUS(NDM%AL(JP), 0, 0, DM%LPROSNOW)
   ENDDO
ENDIF
!
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N',1,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
CONTAINS
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_MISC_BUD(DMA, KLUA, KLUAP)
!
TYPE(DIAG_MISC_ISBA_t), INTENT(INOUT) :: DMA
!
INTEGER, INTENT(IN) :: KLUA
INTEGER, INTENT(IN) :: KLUAP
!
INTEGER          :: INOPATCH
INTEGER          :: ISNLAYER
INTEGER          :: ISW
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_BUD',0,ZHOOK_HANDLE)
!
!////////////DIAG DEFINED BY PATCH AND AVERAGED//////////
!
!* General case
!
ALLOCATE(DMA%XSWI   (KLUA,IO%NGROUND_LAYER))
ALLOCATE(DMA%XTSWI  (KLUA,IO%NGROUND_LAYER))
!
DMA%XSWI  = XUNDEF
DMA%XTSWI = XUNDEF
!
ALLOCATE(DMA%XHV       (KLUA))
ALLOCATE(DMA%XTWSNOW   (KLUA))
ALLOCATE(DMA%XTDSNOW   (KLUA))
ALLOCATE(DMA%XTTSNOW   (KLUA))
ALLOCATE(DMA%XTSNOWAGE (KLUA))
ALLOCATE(DMA%XTSNOWLIQ (KLUA))
ALLOCATE(DMA%XPSNG     (KLUA))
ALLOCATE(DMA%XPSNV     (KLUA))
ALLOCATE(DMA%XPSN      (KLUA))
ALLOCATE(DMA%XFSAT     (KLUA)) 
ALLOCATE(DMA%XFFG      (KLUA))
ALLOCATE(DMA%XFFV      (KLUA))
ALLOCATE(DMA%XFF       (KLUA))
ALLOCATE(DMA%XF2       (KLUA))
ALLOCATE(DMA%XSOIL_SWI (KLUA))
ALLOCATE(DMA%XSOIL_TSWI(KLUA))
ALLOCATE(DMA%XSOIL_TWG (KLUA))
ALLOCATE(DMA%XSOIL_TWGI(KLUA))
ALLOCATE(DMA%XSOIL_WG  (KLUA))
ALLOCATE(DMA%XSOIL_WGI (KLUA))
ALLOCATE(DMA%XROOT_TWG (KLUA))
ALLOCATE(DMA%XSURF_TWG (KLUA))
ALLOCATE(DMA%XGFLUXSNOW(KLUA))
!
DMA%XHV          = XUNDEF  
DMA%XSWI         = XUNDEF
DMA%XTSWI        = XUNDEF
DMA%XTWSNOW      = XUNDEF
DMA%XTDSNOW      = XUNDEF
DMA%XTTSNOW      = XUNDEF  
DMA%XTSNOWAGE    = XUNDEF  
DMA%XTSNOWLIQ    = XUNDEF  
DMA%XPSNG        = XUNDEF
DMA%XPSNV        = XUNDEF
DMA%XPSN         = XUNDEF
DMA%XFSAT        = XUNDEF  
DMA%XFFG         = XUNDEF
DMA%XFFV         = XUNDEF
DMA%XFF          = XUNDEF
DMA%XF2          = XUNDEF
DMA%XSOIL_TSWI   = XUNDEF
DMA%XSOIL_SWI    = XUNDEF
DMA%XSOIL_TWG    = XUNDEF
DMA%XSOIL_TWGI   = XUNDEF
DMA%XSOIL_WG     = XUNDEF
DMA%XSOIL_WGI    = XUNDEF
DMA%XROOT_TWG    = XUNDEF
DMA%XSURF_TWG    = XUNDEF
DMA%XGFLUXSNOW   = XUNDEF
!
!* ISBA DIF case
!
IF(IO%CISBA=='DIF')THEN
  !
  ALLOCATE(DMA%XALT   (KLUA))
  ALLOCATE(DMA%XFLT   (KLUA))
  ALLOCATE(DMA%XPLT   (KLUA))
  ALLOCATE(DMA%XF2WGHT(KLUA,IO%NGROUND_LAYER))
  !
  DMA%XALT    = XUNDEF
  DMA%XFLT    = XUNDEF        
  DMA%XPLT    = XUNDEF        
  DMA%XF2WGHT = XUNDEF
  !
ELSE
  !
  ALLOCATE(DMA%XALT   (0))
  ALLOCATE(DMA%XFLT   (0))
  ALLOCATE(DMA%XPLT   (0))
  ALLOCATE(DMA%XF2WGHT(0,0))
  !
ENDIF
!
!* Specific ISBA-CC
!
IF(IO%CRESPSL=='CNT'.OR.IO%CRESPSL=='DIF')THEN
  ALLOCATE(DMA%XTSOILPOOL(KLUA,IO%NNSOILCARB))
  DMA%XTSOILPOOL = XUNDEF
ELSE
  ALLOCATE(DMA%XTSOILPOOL(0,0))
ENDIF
!
IF(IO%LFIRE)THEN
  ALLOCATE(DMA%XFIREFRA(KLUA))
  DMA%XFIREFRA = XUNDEF
ELSE
  ALLOCATE(DMA%XFIREFRA(0))
ENDIF
!
!* Soil gas scheme
!
IF(IO%LSOILGAS)THEN
  !
  ALLOCATE(DMA%XSOILO2  (KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XSOILCO2 (KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XSOILCH4 (KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XOXIC_O2 (KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XOXIC_CO2(KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XMG_CH4  (KLUA,IO%NGROUND_LAYER))
  ALLOCATE(DMA%XMT_CH4  (KLUA,IO%NGROUND_LAYER))
  !
  !
ELSE
  !
  ALLOCATE(DMA%XSOILO2  (0,0))
  ALLOCATE(DMA%XSOILCO2 (0,0))
  ALLOCATE(DMA%XSOILCH4 (0,0))
  ALLOCATE(DMA%XOXIC_O2 (0,0))
  ALLOCATE(DMA%XOXIC_CO2(0,0))
  ALLOCATE(DMA%XMG_CH4  (0,0))
  ALLOCATE(DMA%XMT_CH4  (0,0))
  !        
ENDIF
!
!* Specific soil and snow Nudging
!
IF(IO%CNUDG_WG/='DEF')THEN
  ALLOCATE(DMA%XNUDGINCSM (KLUA))
  ALLOCATE(DMA%XNUDGINCSML(KLUA,IO%NGROUND_LAYER))
  DMA%XNUDGINCSM  = XUNDEF
  DMA%XNUDGINCSML = XUNDEF
ELSE
  ALLOCATE(DMA%XNUDGINCSM (0)  )
  ALLOCATE(DMA%XNUDGINCSML(0,0))
ENDIF
!
IF(IO%LNUDG_SWE)THEN
  ALLOCATE(DMA%XNUDGINCSWE(KLUA))
  DMA%XNUDGINCSWE = XUNDEF
ELSE
  ALLOCATE(DMA%XNUDGINCSWE(0))
ENDIF
!
!//////////////DIAG DEFINED ONLY BY PATCH//////////////
!//////////////          KLUAP>0        ///////////////
!
!
IF(KLUAP>0)THEN
  ISNLAYER = KSNOW_NLAYER
  ISW      = KSW
ELSE
  ISNLAYER = 0
  ISW      = 0
ENDIF
!
ALLOCATE(DMA%XSNOWLIQ   (KLUAP,ISNLAYER))
ALLOCATE(DMA%XSNOWTEMP  (KLUAP,ISNLAYER))
ALLOCATE(DMA%XSNOWDZ    (KLUAP,ISNLAYER))
ALLOCATE(DMA%XSPEC_ALB  (KLUAP,ISW     ))
ALLOCATE(DMA%XDIFF_RATIO(KLUAP,ISW     ))
!
IF (KLUAP>0) THEN
  DMA%XSNOWLIQ    = XUNDEF
  DMA%XSNOWTEMP   = XUNDEF
  DMA%XSNOWDZ     = XUNDEF
  DMA%XSPEC_ALB   = XUNDEF
  DMA%XDIFF_RATIO = XUNDEF
ENDIF
!
IF (IO%LTR_ML) THEN
  !
  ALLOCATE (DMA%XFAPAR    (KLUAP))
  ALLOCATE (DMA%XFAPIR    (KLUAP))
  ALLOCATE (DMA%XFAPAR_BS (KLUAP))
  ALLOCATE (DMA%XFAPIR_BS (KLUAP))
  ALLOCATE (DMA%XDFAPARC  (KLUAP))
  ALLOCATE (DMA%XDFAPIRC  (KLUAP))
  ALLOCATE (DMA%XDLAI_EFFC(KLUAP))
  !
  IF (KLUAP>0) THEN
    DMA%XFAPAR      = XUNDEF
    DMA%XFAPIR      = XUNDEF
    DMA%XFAPAR_BS   = XUNDEF
    DMA%XFAPIR_BS   = XUNDEF
    DMA%XDFAPARC    = 0.
    DMA%XDFAPIRC    = 0.
    DMA%XDLAI_EFFC  = 0.
  ENDIF
  !
ELSE
  !
  ALLOCATE (DMA%XFAPAR      (0))
  ALLOCATE (DMA%XFAPIR      (0))
  ALLOCATE (DMA%XFAPAR_BS   (0))
  ALLOCATE (DMA%XFAPIR_BS   (0))
  ALLOCATE (DMA%XDFAPARC    (0))
  ALLOCATE (DMA%XDFAPIRC    (0))
  ALLOCATE (DMA%XDLAI_EFFC  (0))
  !
ENDIF
!
ALLOCATE(DMA%XC1       (KLUAP))
ALLOCATE(DMA%XC2       (KLUAP))
ALLOCATE(DMA%XWGEQ     (KLUAP))
ALLOCATE(DMA%XCG       (KLUAP))
ALLOCATE(DMA%XCT       (KLUAP))
ALLOCATE(DMA%XRS       (KLUAP))
ALLOCATE(DMA%XSNOWHMASS(KLUAP))
ALLOCATE(DMA%XSRSFC    (KLUAP))
ALLOCATE(DMA%XRRSFC    (KLUAP))
ALLOCATE(DMA%XRNSNOW   (KLUAP))
ALLOCATE(DMA%XHSNOW    (KLUAP))
ALLOCATE(DMA%XHPSNOW   (KLUAP))
ALLOCATE(DMA%XUSTARSNOW(KLUAP))
ALLOCATE(DMA%XCDSNOW   (KLUAP))
ALLOCATE(DMA%XCHSNOW   (KLUAP))
!
IF (KLUAP>0) THEN
  DMA%XC1        = XUNDEF
  DMA%XC2        = XUNDEF
  DMA%XWGEQ      = XUNDEF
  DMA%XCG        = XUNDEF
  DMA%XCT        = XUNDEF
  DMA%XRS        = XUNDEF
  DMA%XSNOWHMASS = XUNDEF
  DMA%XSRSFC     = XUNDEF
  DMA%XRRSFC     = XUNDEF
  DMA%XRNSNOW    = XUNDEF
  DMA%XHSNOW     = XUNDEF
  DMA%XHPSNOW    = XUNDEF
  DMA%XUSTARSNOW = XUNDEF
  DMA%XCDSNOW    = XUNDEF
  DMA%XCHSNOW    = XUNDEF
ENDIF
!
!
!//////////////DIAG DEFINED ONLY AVERAGED//////////////
!//////////////        KLUA=KLUAP        //////////////
!
IF(KLUA==KLUAP)THEN
   INOPATCH=0
ELSE
   INOPATCH=KLUA
ENDIF
!
ALLOCATE(DMA%XWR  (INOPATCH))
ALLOCATE(DMA%XWRVN(INOPATCH))
ALLOCATE(DMA%XLAI (INOPATCH))
!
IF (INOPATCH>0) THEN
  DMA%XWR          = XUNDEF        
  DMA%XWRVN        = XUNDEF        
  DMA%XLAI         = XUNDEF        
ENDIF
!
IF(IO%CISBA=='DIF'.AND.DM%LSURF_MISC_DIF)THEN
  !
  ALLOCATE(DMA%XFRD2_TSWI(INOPATCH))
  ALLOCATE(DMA%XFRD2_TWG (INOPATCH))
  ALLOCATE(DMA%XFRD2_TWGI(INOPATCH))
  ALLOCATE(DMA%XFRD3_TSWI(INOPATCH))
  ALLOCATE(DMA%XFRD3_TWG (INOPATCH))
  ALLOCATE(DMA%XFRD3_TWGI(INOPATCH))
  !
  IF (INOPATCH>0) THEN  
    DMA%XFRD2_TSWI = XUNDEF
    DMA%XFRD2_TWG  = XUNDEF
    DMA%XFRD2_TWGI = XUNDEF
    DMA%XFRD3_TSWI = XUNDEF
    DMA%XFRD3_TWG  = XUNDEF
    DMA%XFRD3_TWGI = XUNDEF 
  ENDIF 
  !
ELSE
  !
  ALLOCATE(DMA%XFRD2_TSWI(0))
  ALLOCATE(DMA%XFRD2_TWG (0))
  ALLOCATE(DMA%XFRD2_TWGI(0))
  ALLOCATE(DMA%XFRD3_TSWI(0))
  ALLOCATE(DMA%XFRD3_TWG (0))
  ALLOCATE(DMA%XFRD3_TWGI(0))          
  !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_BUD',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_MISC_BUD
!
!
!-------------------------------------------------------------------------------
!
!
SUBROUTINE ALLOC_MISC_CROCUS(DMA, KLUA, KLUAP, OPROSNOW)
!
TYPE(DIAG_MISC_ISBA_t), INTENT(INOUT) :: DMA
!
INTEGER, INTENT(IN) :: KLUA
INTEGER, INTENT(IN) :: KLUAP
LOGICAL, INTENT(IN) :: OPROSNOW
!
INTEGER         :: ISNLAYER
INTEGER         :: ISW
INTEGER         :: IIMPUR
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_CROCUS',0,ZHOOK_HANDLE)
!
IF(KLUAP>0)THEN
  ISNLAYER = KSNOW_NLAYER
  ISW      = KSW
  IIMPUR   = NIMPUR
ELSE
  ISNLAYER = 0
  ISW      = 0
  IIMPUR   = 0
ENDIF
!
! * PROSNOW only per patch
!
IF ( OPROSNOW ) THEN
   !
   ALLOCATE(DMA%XSNOWDEND      (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWSPHER     (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWSIZE      (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWSSA       (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWTYPEMEPRA (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWRAM       (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSNOWSHEAR     (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XACC_RAT       (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XNAT_RAT       (KLUAP,ISNLAYER))
   ALLOCATE(DMA%XSPEC_ALB      (KLUAP,ISW))
   ALLOCATE(DMA%XDIFF_RATIO    (KLUAP,ISW))
   ALLOCATE(DMA%XIMPUR_CONC    (KLUAP,ISNLAYER,IIMPUR))
   !
   IF (KLUAP>0) THEN
      DMA%XSNOWDEND      = XUNDEF
      DMA%XSNOWSPHER     = XUNDEF
      DMA%XSNOWSIZE      = XUNDEF
      DMA%XSNOWSSA       = XUNDEF
      DMA%XSNOWTYPEMEPRA = XUNDEF
      DMA%XSNOWRAM       = XUNDEF
      DMA%XSNOWSHEAR     = XUNDEF
      DMA%XACC_RAT       = XUNDEF
      DMA%XNAT_RAT       = XUNDEF
      DMA%XSPEC_ALB      = XUNDEF
      DMA%XDIFF_RATIO    = XUNDEF
      DMA%XIMPUR_CONC    = XUNDEF  
   ENDIF
   !
   ALLOCATE(DMA%XSNDPT_1DY     (KLUA))
   ALLOCATE(DMA%XSNDPT_3DY     (KLUA))
   ALLOCATE(DMA%XSNDPT_5DY     (KLUA))
   ALLOCATE(DMA%XSNDPT_7DY     (KLUA))
   ALLOCATE(DMA%XSNSWE_1DY     (KLUA))
   ALLOCATE(DMA%XSNSWE_3DY     (KLUA))
   ALLOCATE(DMA%XSNSWE_5DY     (KLUA))
   ALLOCATE(DMA%XSNSWE_7DY     (KLUA))
   ALLOCATE(DMA%XSNRAM_SONDE   (KLUA))
   ALLOCATE(DMA%XSN_WETTHCKN   (KLUA))
   ALLOCATE(DMA%XSN_REFRZNTHCKN(KLUA))
   ALLOCATE(DMA%XDEP_HIG       (KLUA))
   ALLOCATE(DMA%XDEP_MOD       (KLUA))
   ALLOCATE(DMA%XACC_LEV       (KLUA))
   ALLOCATE(DMA%XPRO_INF_TYP   (KLUA))
   !
   IF (KLUA>0) THEN
      DMA%XSNDPT_1DY      = XUNDEF
      DMA%XSNDPT_3DY      = XUNDEF
      DMA%XSNDPT_5DY      = XUNDEF
      DMA%XSNDPT_7DY      = XUNDEF
      DMA%XSNSWE_1DY      = XUNDEF
      DMA%XSNSWE_3DY      = XUNDEF
      DMA%XSNSWE_5DY      = XUNDEF
      DMA%XSNSWE_7DY      = XUNDEF
      DMA%XSNRAM_SONDE    = XUNDEF
      DMA%XSN_WETTHCKN    = XUNDEF
      DMA%XSN_REFRZNTHCKN = XUNDEF
      DMA%XDEP_HIG        = XUNDEF
      DMA%XDEP_MOD        = XUNDEF
      DMA%XACC_LEV        = XUNDEF
      DMA%XPRO_INF_TYP    = XUNDEF
   ENDIF
   !
ELSE
   !
   ALLOCATE(DMA%XSNOWDEND      (0,0))
   ALLOCATE(DMA%XSNOWSPHER     (0,0))
   ALLOCATE(DMA%XSNOWSIZE      (0,0))
   ALLOCATE(DMA%XSNOWSSA       (0,0))
   ALLOCATE(DMA%XSNOWTYPEMEPRA (0,0))
   ALLOCATE(DMA%XSNOWRAM       (0,0))
   ALLOCATE(DMA%XSNOWSHEAR     (0,0))
   ALLOCATE(DMA%XACC_RAT       (0,0))
   ALLOCATE(DMA%XNAT_RAT       (0,0))
   !
   ALLOCATE(DMA%XSNDPT_1DY     (0))
   ALLOCATE(DMA%XSNDPT_3DY     (0))
   ALLOCATE(DMA%XSNDPT_5DY     (0))
   ALLOCATE(DMA%XSNDPT_7DY     (0))
   ALLOCATE(DMA%XSNSWE_1DY     (0))
   ALLOCATE(DMA%XSNSWE_3DY     (0))
   ALLOCATE(DMA%XSNSWE_5DY     (0))
   ALLOCATE(DMA%XSNSWE_7DY     (0))
   ALLOCATE(DMA%XSNRAM_SONDE   (0))
   ALLOCATE(DMA%XSN_WETTHCKN   (0))
   ALLOCATE(DMA%XSN_REFRZNTHCKN(0))
   ALLOCATE(DMA%XDEP_HIG       (0))
   ALLOCATE(DMA%XDEP_MOD       (0))
   ALLOCATE(DMA%XACC_LEV       (0))
   ALLOCATE(DMA%XPRO_INF_TYP   (0))
   !
   ALLOCATE(DMA%XIMPUR_CONC    (0,0,0)) 
   ALLOCATE(DMA%XSPEC_ALB      (0,0))
   ALLOCATE(DMA%XDIFF_RATIO    (0,0))   
   !
ENDIF
!
! * SYTRON
!
IF ( IO%LSNOWSYTRON ) THEN
   !
   ALLOCATE(DMA%XSYTMASS (KLUA))
   ALLOCATE(DMA%XSYTMASSC(KLUA))
   !
   IF (KLUA>0) THEN
      DMA%XSYTMASS = XUNDEF
      DMA%XSYTMASSC= 0.
   ENDIF
   !
ELSE
   !
   ALLOCATE(DMA%XSYTMASS (0))
   ALLOCATE(DMA%XSYTMASSC(0))
   !
ENDIF
!
! * SNOWMAK_BOOL
!
IF (IO%LSNOWMAK_BOOL ) THEN
   !
   ALLOCATE(DMA%XPRODCOUNT(KLUA))
   !
   IF (KLUA>0) THEN
     DMA%XPRODCOUNT = XUNDEF
   ENDIF
   !
ELSE
   !
   ALLOCATE(DMA%XPRODCOUNT(0))
   !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_CROCUS',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_MISC_CROCUS
!
!
!-------------------------------------------------------------------------------
!
!
SUBROUTINE ALLOC_MISC_UTCI(DUA, KLUA)
!
TYPE(DIAG_UTCI_t), INTENT(INOUT) :: DUA
!
INTEGER, INTENT(IN) :: KLUA
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_UTCI',0,ZHOOK_HANDLE)
!
ALLOCATE(DUA%XUTCI_OUTSUN       (KLUA))
ALLOCATE(DUA%XUTCI_OUTSHADE     (KLUA))
ALLOCATE(DUA%XUTCI_OUTSUN_MEAN  (KLUA))
ALLOCATE(DUA%XUTCI_OUTSHADE_MEAN(KLUA))
ALLOCATE(DUA%XTRAD_SUN          (KLUA))
ALLOCATE(DUA%XTRAD_SHADE        (KLUA))
ALLOCATE(DUA%XTRAD_SUN_MEAN     (KLUA))
ALLOCATE(DUA%XTRAD_SHADE_MEAN   (KLUA))
!
ALLOCATE(DUA%XUTCIC_OUTSUN      (KLUA,NUTCI_STRESS))
ALLOCATE(DUA%XUTCIC_OUTSHADE    (KLUA,NUTCI_STRESS))
!
IF (KLUA>0) THEN
  DUA%XUTCI_OUTSUN        = XUNDEF
  DUA%XUTCI_OUTSHADE      = XUNDEF
  DUA%XUTCI_OUTSUN_MEAN   = 0.0
  DUA%XUTCI_OUTSHADE_MEAN = 0.0
  DUA%XTRAD_SUN           = XUNDEF
  DUA%XTRAD_SHADE         = XUNDEF
  DUA%XTRAD_SUN_MEAN      = 0.0
  DUA%XTRAD_SHADE_MEAN    = 0.0
  DUA%XUTCIC_OUTSUN       = 0.
  DUA%XUTCIC_OUTSHADE     = 0.
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_MISC_ISBA_INIT_N:ALLOC_MISC_UTCI',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_MISC_UTCI
!
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE DIAG_MISC_ISBA_INIT_n
